package cz.cuni.amis.pogamut.emohawk.examples.emptybot;

import SteeringProperties.SteeringProperties;
import SteeringProperties.TargetApproachingProperties;
import cz.cuni.amis.introspection.java.JProp;
import cz.cuni.amis.pogamut.base.agent.impl.AgentId;
import cz.cuni.amis.pogamut.base.utils.guice.AgentScoped;
import cz.cuni.amis.pogamut.base3d.worldview.object.Location;
import cz.cuni.amis.pogamut.emohawk.agent.module.sensomotoric.*;
import cz.cuni.amis.pogamut.emohawk.bot.impl.EmohawkBotController;
import cz.cuni.amis.pogamut.ut2004.agent.module.sensor.AgentInfo;
import cz.cuni.amis.pogamut.ut2004.bot.impl.UT2004Bot;
import cz.cuni.amis.pogamut.ut2004.bot.params.UT2004BotParameters;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbcommands.Initialize;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbcommands.SendMessage;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.BotKilled;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.ConfigChange;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.GameInfo;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.InitedMessage;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.Self;
import cz.cuni.amis.pogamut.ut2004.communication.parser.UT2004Parser;
import cz.cuni.amis.pogamut.ut2004.utils.UT2004BotRunner;
import cz.cuni.amis.utils.exception.PogamutException;
import java.util.logging.Level;

/**
 * Pogamut's "Hello world!" example showing few extra things such as introspection
 * and various bot-initializing methods.
 * <p><p>
 * First, try to run the bot and kill it... than uncomment the line inside {@link EmohawkEmptyBot#botKilled(BotKilled)} 
 * and run the bot again kill it to see the difference. 
 *
 * @author Michal Bida aka Knight
 * @author Rudolf Kadlec aka ik
 * @author Jakub Gemrot aka Jimmy
 */
@AgentScoped
public class EmohawkEmptyBot extends EmohawkBotController {

    @JProp
    public String stringProp = "Hello bot example";
    @JProp
    public boolean boolProp = true;
    @JProp
    public int intProp = 2;
    @JProp
    public double doubleProp = 1.0;

    /**
     * Initialize all necessary variables here, before the bot actually receives anything
     * from the environment.
     */
    @Override
    public void prepareBot(UT2004Bot bot) {
        
    }
    
    /**
     * Here we can modify initializing command for our bot, e.g., sets its name or skin.
     * @return instance of {@link Initialize}
     */
    @Override
    public Initialize getInitializeCommand() {
        return new Initialize().setName(this.getBot().getParams().getAgentId().getName().getFlag());//.setClassName(CharacterType.UDN_MALE.getUE2Class());
    }
    
    /**
     * Handshake with GameBots2004 is over - bot has information about the map in its world view.
     * Many agent modules are usable since this method is called.
     * @param gameInfo informaton about the game type
     * @param config information about configuration
     * @param init information about configuration
     */
    @Override
    public void botInitialized(GameInfo gameInfo, ConfigChange currentConfig, InitedMessage init) {
        ((UT2004Parser)bot.getEventBus().getComponent(UT2004Parser.COMPONENT_ID)).getLog().setLevel(Level.OFF);
        bot.getLogger().getCategory("Yylex").setLevel(Level.OFF);
    }

    /**
     * The bot is initilized in the environment - a physical representation of the bot is present in the game.
     * @param gameInfo informaton about the game type
     * @param config information about configuration
     * @param init information about configuration
     * @param self information about the agent
     */
    @Override
    public void botFirstSpawn(GameInfo gameInfo, ConfigChange config, InitedMessage init, Self self) {
        // Display a welcome message in the game engine
    	// right in the time when the bot appears in the environment, i.e., his body has just been spawned 
    	// into the UT2004 for the first time.
        comm.sendGlobalTextMessage("Hello world! I am alive!");
        
        // alternatively, you may use getAct() method for issuing arbitrary {@link CommandMessage} for the bot's body
        // inside UT2004
        getAct().act(new SendMessage().setGlobal(true).setText("And I can speak! Hurray!"));
    }
    
    /**
     * This method is called only once right before actual logic() method is called for the first time.
     */
    @Override
    public void beforeFirstLogic() {
    }

    /**
     * Main method that controls the bot - makes decisions what to do next.
     * It is called iteratively by Pogamut engine every time a synchronous batch
     * from the environment is received. This is usually 4 times per second - it
     * is affected by visionTime variable, that can be adjusted in GameBots ini file in
     * UT2004/System folder.
     *
     * @throws cz.cuni.amis.pogamut.base.exceptions.PogamutException
     */
    @Override
    public void logic() throws PogamutException {
        // Log example message this won't be shown in the game, it is just for debugging purposes
    	// The log is written to the console output if the log level is low enough (i.e., at least Level.INFO)
        if (boolProp) {
        	log.info(stringProp + ": still working.");
        }
        //this.comm.sendGlobalTextMessage("logic");

        // Increment the example property, you should observe in the IDE how 
        // fast it is being incremented
        intProp++;
    }

    /**
     * Called each time the bot dies. Good for reseting all bot's state dependent variables.
     *
     * @param event
     */
    @Override
    public void botKilled(BotKilled event) {
    	// First, try to run the bot and kill it... than uncomment this line and run the bot again
    	// kill it and see the difference.
        //body.getCommunication().sendGlobalTextMessage("I was KILLED!");
    }

    /**
     * This method is called when the bot is started either from IDE or from command line.
     *
     * @param args
     */
    public static void main(String args[]) throws PogamutException {
    	// wrapped logic for bots executions, suitable to run single bot in single JVM
        new UT2004BotRunner<UT2004Bot, UT2004BotParameters>(EmohawkEmptyBot.class, "Emohawk-Bot").setPort(3000).setMain(true).setLogLevel(Level.WARNING).startAgents(
            new UT2004BotParameters().setAgentId(new AgentId("EmohawkBotOne")),
            new UT2004BotParameters().setAgentId(new AgentId("EmohawkBotTwo"))
        );
    }
}
